"""Module to run Tc calculation."""
from jarvis.io.qe.outputs import DataFileSchema
from jarvis.tasks.qe.qe import QEjob


def supercond_workflow(atoms=None, kp=None):
    """Calculate Tc using QE."""
    # Still under development,
    relax = {
        "control": {
            "calculation": "'vc-relax'",
            "restart_mode": "'from_scratch'",
            "prefix": "'QE'",
            "outdir": "'./'",
            "tstress": ".true.",
            "tprnfor": ".true.",
            "disk_io": "'low'",
            "wf_collect": ".true.",
            "pseudo_dir": None,
            "verbosity": "'high'",
            "nstep": 100,
        },
        "system": {
            "ibrav": 0,
            "nat": None,
            "ntyp": None,
            "ecutwfc": 45,
            "ecutrho": 250,
            "q2sigma": 1,
            "ecfixed": 44.5,
            "qcutz": 800,
            "occupations": "'smearing'",
            "degauss": 0.01,
            "lda_plus_u": ".false.",
            "force_symmorphic": ".true.",
            "nosym": ".false.",
            "noinv": ".false.",
        },
        "electrons": {
            "diagonalization": "'david'",
            "mixing_mode": "'local-TF'",
            "mixing_beta": 0.3,
            "conv_thr": "1d-9",
        },
        "ions": {"ion_dynamics": "'bfgs'"},
        "cell": {"cell_dynamics": "'bfgs'", "cell_dofree": "'all'"},
    }

    qejob_relax = QEjob(
        atoms=atoms,
        input_params=relax,
        output_file="relax.out",
        qe_cmd="/users/knc6/Software/QE/q-e/bin/pw.x",
        jobname="relax",
        kpoints=kp,
        input_file="arelax.in",
    )

    info = qejob_relax.runjob()
    xml_path = info["xml_path"]
    xml_dat = DataFileSchema(xml_path)
    final_strt = xml_dat.final_structure

    print("final_strt", final_strt)

    scf = {
        "control": {
            "calculation": "'scf'",
            "restart_mode": "'from_scratch'",
            "prefix": "'QE'",
            "outdir": "'./'",
            "tstress": ".true.",
            "tprnfor": ".true.",
            "disk_io": "'low'",
            "pseudo_dir": None,
            "verbosity": "'high'",
            "nstep": 100,
            "etot_conv_thr": "1.0d-5",
            "forc_conv_thr": "1.0d-4",
        },
        "system": {
            "ibrav": 0,
            "degauss": 0.01,
            "nat": None,
            "ntyp": None,
            "ecutwfc": 45,
            "ecutrho": 250,
            "occupations": "'smearing'",
            "smearing": "'mp'",
            "la2F ": ".true.",
        },
        "electrons": {
            "diagonalization": "'david'",
            "mixing_mode": "'plain'",
            "mixing_beta": 0.7,
            "conv_thr": "1d-9",
        },
    }

    qejob_scf = QEjob(
        atoms=final_strt,
        input_params=scf,
        output_file="scf.out",
        qe_cmd="/users/knc6/Software/QE/q-e/bin/pw.x",
        jobname="scf",
        kpoints=kp,
        input_file="ascf.in",
    )

    qejob_scf.runjob()
    ph = {
        "inputph": {
            "prefix": "'QE'",
            "fildyn": "'QE.dyn'",
            "outdir": "'./'",
            "ldisp": ".true.",
            "trans": ".true.",
            "fildvscf": "'dvscf'",
            "electron_phonon": "'interpolated'",
            "nq1": 2,
            "nq2": 2,
            "nq3": 3,
            "tr2_ph": "1.0d-14",
        }
    }
    qejob_ph = QEjob(
        atoms=atoms,
        input_params=ph,
        output_file="ph.out",
        qe_cmd="/users/knc6/Software/QE/q-e/bin/ph.x",
        jobname="ph",
        kpoints=None,
        input_file="aph.in",
    )

    qejob_ph.runjob()

    qr = {
        "input": {
            "zasr": "'simple'",
            "fildyn": "'QE.dyn'",
            "flfrc": "'QE333.fc'",
            "la2F": ".true.",
        }
    }
    qejob_qr = QEjob(
        atoms=atoms,
        input_params=qr,
        output_file="q2r.out",
        qe_cmd="/users/knc6/Software/QE/q-e/bin/q2r.x",
        jobname="qr",
        kpoints=None,
        input_file="aqr.in",
    )

    qejob_qr.runjob()

    matdyn = {
        "input": {
            "asr": "'simple'",
            "flfrc": "'QE333.fc'",
            "flfrq": "'QE333.freq'",
            "la2F": ".true.",
            "dos": ".true.",
            "fldos": "'phonon.dos'",
            "nk1": 10,
            "nk2": 10,
            "nk3": 10,
            "ndos": 50,
        }
    }

    qejob_matdyn = QEjob(
        atoms=atoms,
        input_params=matdyn,
        output_file="matdyn.out",
        qe_cmd="/users/knc6/Software/QE/q-e/bin/matdyn.x",
        jobname="matdyn",
        kpoints=None,
        input_file="amatdyn.in",
    )

    qejob_matdyn.runjob()


"""
if __name__ == "__main__":
    from jarvis.db.jsonutils import loadjson, dumpjson
    from jarvis.core.atoms import Atoms
    from jarvis.db.figshare import get_jid_data
    from jarvis.core.kpoints import Kpoints3D

    dat = get_jid_data(jid="JVASP-19821", dataset="dft_3d")
    atoms = Atoms.from_dict(dat["atoms"])
    kp = Kpoints3D().automatic_length_mesh(
        lattice_mat=atoms.lattice_mat, length=int(dat["kpoint_length_unit"])
    )
    print("kpoint_length_unit", dat["kpoint_length_unit"], kp)
    print(atoms)
    supercond_workflow(atoms=atoms, kp=kp)
"""



from jarvis.analysis.defects.vacancy import Vacancy
from jarvis.analysis.structure.spacegroup import Spacegroup3D
from jarvis.db.jsonutils import dumpjson
from jarvis.core.atoms import Atoms
from jarvis.tasks.vasp.vasp import VaspJob, write_vaspjob
from jarvis.tasks.queue_jobs import Queue
from jarvis.io.vasp.inputs import Poscar
from jarvis.db.figshare import data
from jarvis.io.vasp.inputs import Poscar, Incar, Potcar
from jarvis.core.kpoints import Kpoints3D

dat = data("dft_3d")
import os

data0 = dict(
    PREC="Accurate",
    ISMEAR=0,
    GGA="BO",
    PARAM1=0.1833333333,
    PARAM2=0.2200000000,
    LUSE_VDW=".TRUE.",
    AGGAC=0.0000,
    EDIFF="1E-7",
    NELM=400,
    ISPIN=2,
    # NPAR=32,
    POTIM=0.02,
    LCHARG=".FALSE.",
    LVTOT=".FALSE.",
    LVHAR=".FALSE.",
    LWAVE=".TRUE.",
)

data1 = dict(
    PREC="Accurate",
    ISMEAR=0,
    IBRION=6,
    GGA="BO",
    PARAM1=0.1833333333,
    PARAM2=0.2200000000,
    LUSE_VDW=".TRUE.",
    AGGAC=0.0000,
    EDIFF="1E-7",
    NELM=400,
    ISPIN=2,
    # NPAR=32,
    POTIM=0.02,
    LCHARG=".FALSE.",
    LVTOT=".FALSE.",
    LVHAR=".FALSE.",
    LWAVE=".FALSE.",
)

data2 = dict(
    PREC="Accurate",
    ISMEAR=-2,
    IBRION=6,
    GGA="BO",
    PARAM1=0.1833333333,
    PARAM2=0.2200000000,
    LUSE_VDW=".TRUE.",
    AGGAC=0.0000,
    EDIFF="1E-7",
    NELM=400,
    ISPIN=2,
    # NPAR=32,
    POTIM=0.02,
    LCHARG=".FALSE.",
    LVTOT=".FALSE.",
    LVHAR=".FALSE.",
    LWAVE=".FALSE.",
)

inc0 = Incar(data0)
inc1 = Incar(data1)
inc2 = Incar(data2)


def get_symb(atoms):
    new_symb = []
    for i in atoms.elements:
        if i not in new_symb:
            new_symb.append(i)
    return new_symb


#vasp_cmd = "/usr/bin/mpirun /usr/local/src/vasp-5.4.4/build/std/vasp"
#vasp_cmd = "/usr/bin/mpirun /users/knc6/VASP/vasp544/vasp.5.4.4/bin/vasp_std"
#copy_files = ["/users/knc6/bin/vdw_kernel.bindat"]
jids = ["JVASP-19821"]  # ,'JVASP-14615']

jids = [
    "JVASP-93855",
    "JVASP-19821",
    "JVASP-19668",
    "JVASP-19684",
    "JVASP-54933",
]
jids = ["JVASP-1002"]  # ,'JVASP-14615']


def supercon_igor(
    id="JVASP-1002",
    atoms=None,
    copy_files=["/users/knc6/bin/vdw_kernel.bindat"],
    vasp_cmd="/usr/bin/mpirun /users/knc6/VASP/vasp544/vasp.5.4.4/bin/vasp_std",
):
    if "JVASP-" in id:
        for i in dat:
            if i["jid"] == id:
                atoms = Atoms.from_dict(i["atoms"])

    strt = Spacegroup3D(atoms).conventional_standard_structure
    encut = i["encut"]

    # STEP1: Generate WAVECAR

    name = str(id) + "_RUN0"
    wavecar_path = os.getcwd() + "/" + name + "/" + name + "/WAVECAR"
    print("WAVECAR path", wavecar_path)
    pos = Poscar(strt)
    symb = get_symb(strt)
    inc0.update({"ENCUT": i["encut"]})
    pos.comment = name
    pot = Potcar(elements=symb)
    kp = (
        Kpoints3D()
        .automatic_length_mesh(
            lattice_mat=strt.lattice_mat, length=i["kpoint_length_unit"] -25
        )
        ._kpoints
    )
    kpoints = Kpoints3D(kpoints=kp)

    jobname = name

    job1 = VaspJob(
        poscar=pos,
        incar=inc0,
        potcar=pot,
        kpoints=kpoints,
        vasp_cmd=vasp_cmd,
        copy_files=copy_files,
        jobname=jobname,
    )

    name = str(id) + "_SCREENED"
    # screened
    pos = Poscar(strt)
    symb = get_symb(strt)
    inc1.update({"ENCUT": i["encut"]})
    pos.comment = name
    pot = Potcar(elements=symb)
    kp = (
        Kpoints3D()
        .automatic_length_mesh(
            lattice_mat=strt.lattice_mat, length=i["kpoint_length_unit"] -25
        )
        ._kpoints
    )
    kpoints = Kpoints3D(kpoints=kp)

    jobname = name
    job2 = VaspJob(
        poscar=pos,
        incar=inc1,
        potcar=pot,
        kpoints=kpoints,
        vasp_cmd=vasp_cmd,
        copy_files=copy_files,
        jobname=jobname,
    )

    # break
    name = str(id) + "_UNSCREENED"
    # if not os.path.exists(name):
    #    os.makedirs(name)
    # os.chdir(name)
    # Unscreened
    pos = Poscar(strt)
    symb = get_symb(strt)
    inc2.update({"ENCUT": i["encut"]})
    pos.comment = name
    pot = Potcar(elements=symb)
    kp = (
        Kpoints3D()
        .automatic_length_mesh(
            lattice_mat=strt.lattice_mat, length=i["kpoint_length_unit"] -25
        )
        ._kpoints
    )
    kpoints = Kpoints3D(kpoints=kp)

    jobname = name

    # copy_files.append(wavecar_path)
    job3 = VaspJob(
        poscar=pos,
        incar=inc2,
        potcar=pot,
        kpoints=kpoints,
        vasp_cmd=vasp_cmd,
        copy_files=copy_files,
        jobname=jobname,
    )

    jobs = [job1, job2, job3]
    for ii, i in enumerate(jobs):
        if ii > 0:

            copy_files.append(wavecar_path)
            i.copy_files = copy_files
        name = i.jobname
        if not os.path.exists(name):
            os.makedirs(name)
        os.chdir(name)
        i.runjob()
        """
        job_line = "source ~/anaconda2/envs/my_jarvis/bin/activate my_jarvis \npython job.py \n"
        directory = os.getcwd()

        submit_cmd = ["qsub", "-q", "kamal", "submit_job"]
        Queue.pbs(
            job_line=job_line,
            jobname=name,
            directory=directory,
            cores=32,
            submit_cmd=submit_cmd,
        )
        """
        os.chdir("..")
